﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Unique
{
    class Program
    {
        bool sort = false;
        bool reverse = false;
        System.IO.TextReader fileInput;
        System.IO.TextWriter fileOutput;

        public bool ParseParams(string[] args)
        {
            try
            {
                string input = string.Empty;
                string output = string.Empty;
                int inputIdx = 0;

                if (args.Count() == 0 || args[0].Length == 0)
                {
                    Usage();
                    return false;
                }

                string str = args[0].Substring(0, 1);
                if ((str == "-" || str == "/") && args.Count() > 1)
                {
                    if (args[0].Length < 2)
                    {
                        Usage();
                        return false;
                    }

                    str = args[0].Substring(1, 1).ToUpper();
                    if (str == "S")
                    {
                        sort = true;
                        if (args[0].Length > 2)
                        {
                            str = args[0].Substring(2, 1).ToUpper();
                            if (str != "R")
                            {
                                Usage();
                                return false;
                            }
                            reverse = true;
                        }
                        inputIdx++;
                    }
                    else
                    {
                        Usage();
                        return false;
                    }
                }

                input = args[inputIdx++];
                if (args.Count() > inputIdx)
                {
                    output = args[inputIdx];
                }

                if (!File.Exists(input))
                {
                    Console.WriteLine("Error: could not find input file: '" + input + "'");
                    return false;
                }

                fileInput = new System.IO.StreamReader(input);

                if (output != string.Empty)
                {
                    fileOutput = new System.IO.StreamWriter(output);
                }
                else
                {
                    fileOutput = new StreamWriter(Console.OpenStandardOutput());
                }
                return true;
            }
            catch (Exception /*ex*/)
            {
            }

            return false;
        }

        public void Run()
        {
            List<string> list = new List<string>();
            string line;

            while((line = fileInput.ReadLine()) != null)
            {
                list.Add(line);
            }

            if (sort == true)
            {
                list.Sort();
                if (reverse == true)
                {
                    list.Reverse();
                }
            }

            list = list.Distinct<string>().ToList();

            foreach (string str in list)
            {
                fileOutput.WriteLine(str);
            }

            fileInput.Close();
            fileOutput.Close();
        }

        void Usage()
        {
            Console.WriteLine("Usage: [-h] [-s[r]] <input file> [output file]");
            Console.WriteLine("       -h        (optional) Help - print this message");
            Console.WriteLine("       -s        (optional) Sort the list");
            Console.WriteLine("        r        (optional) Reverse the sort");
            Console.WriteLine("       input     File to read from - required");
            Console.WriteLine("       output    (optional) File to write to, console if omitted");

        }

        static void Main(string[] args)
        {
            Program prog = new Program();

            if (prog.ParseParams(args) == true)
            {
                prog.Run();
            }
        }
    }
}
